package dev.boyfailure.quajra.vectorbreakout;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import javax.swing.JComponent;
import java.awt.geom.Line2D;

public class GameDisplay extends JComponent {
    
    float beamX = 0; // Horizontal location of "vector beam"
    float beamY = 0; // Vertical location of "vector beam"
    
    // Change this to 1 if you build for older versions of Java. Older versions
    // will render a leading zero otherwise. (tested on JDK 1.5)
    byte iStart = 0;
    
    public static float beamScaleX = 1; // Horizontal drawing scale
    public static float beamScaleY = 1; // Vertical drawing scale
    public static float beamThicknessScale = 1; // Thickness of lines
    
    public void paint(Graphics gameGraphics) {
        
        super.paint(gameGraphics);
        
        Graphics2D g2 = (Graphics2D) gameGraphics;
        
        g2.setColor(Color.white);
        
        // Walls
        g2.setColor(Color.gray);
        g2.setStroke(bt(10));
        resetBeam();
        moveBeam(15, 0);
        g2.draw(drawVec(0, 600));
        moveBeam(770, 0);
        g2.draw(drawVec(0, -600));
        moveBeam(15, -5);
        g2.draw(drawVec(-800, 0));
        
        // Bricks
        if (VectorBreakout.currentLevel != null) {
            for (short i = 0; i < VectorBreakout.currentLevel.levelBricks.length; i++) {
                if (VectorBreakout.currentLevel.levelBricks[i] != null) {
                    if (VectorBreakout.currentLevel.levelBricks[i].isBroken == false) {
                        resetBeam();
                        g2.setStroke(bt(3));
                        moveBeam(VectorBreakout.currentLevel.levelBricks[i].brickX + 1, VectorBreakout.currentLevel.levelBricks[i].brickY + 1);
                        g2.setColor(vc(VectorBreakout.currentLevel.levelBricks[i].brickColor));
                        g2.draw(drawVec(VectorBreakout.currentLevel.levelBricks[i].brickWidth - 2, 0));
                        g2.draw(drawVec(0, -(VectorBreakout.currentLevel.levelBricks[i].brickHeight - 2)));
                        g2.draw(drawVec(-(VectorBreakout.currentLevel.levelBricks[i].brickWidth - 2), 0));
                        g2.draw(drawVec(0, VectorBreakout.currentLevel.levelBricks[i].brickHeight - 2));
                    }
                }
            }
        }
        
        // Paddles
        g2.setColor(Color.white);
        for (byte i = 0; i < VectorBreakout.currentPaddles.length; i++) {
            if (VectorBreakout.currentPaddles[i] != null) {
                if (VectorBreakout.currentPaddles[i].isActive) {
                    resetBeam();
                    g2.setStroke(bt(3));
                    moveBeam(VectorBreakout.currentPaddles[i].paddleX, VectorBreakout.currentPaddles[i].paddleY);
                    g2.draw(drawVec(VectorBreakout.currentPaddles[i].paddleWidth, 0));
                    g2.draw(drawVec(0, -(VectorBreakout.currentPaddles[i].paddleHeight)));
                    g2.draw(drawVec(-(VectorBreakout.currentPaddles[i].paddleWidth), 0));
                    g2.draw(drawVec(0, VectorBreakout.currentPaddles[i].paddleHeight));
                }
            }
        }
        
        // Balls
        g2.setStroke(bt(3));
        g2.setColor(Color.yellow);
        for (byte i = 0; i < VectorBreakout.currentBalls.length; i++) {
            if (VectorBreakout.currentBalls[i] != null) {
                if (VectorBreakout.currentBalls[i].isActive) {
                    resetBeam();
                    moveBeam(VectorBreakout.currentBalls[i].ballX, VectorBreakout.currentBalls[i].ballY);
                    g2.draw(drawVec(VectorBreakout.currentBalls[i].ballWidth, 0));
                    g2.draw(drawVec(0, VectorBreakout.currentBalls[i].ballHeight));
                    g2.draw(drawVec(-VectorBreakout.currentBalls[i].ballWidth, 0));
                    g2.draw(drawVec(0, -VectorBreakout.currentBalls[i].ballHeight));
                }
            }
        }
        
        // Particles
        for (byte i = 0; i < VectorBreakout.currentParticles.length; i++) {
            if (VectorBreakout.currentParticles[i] != null) {
                if (VectorBreakout.currentParticles[i].isActive) {
                    resetBeam();
                    moveBeam(VectorBreakout.currentParticles[i].particleX, VectorBreakout.currentParticles[i].particleY);
                    g2.setStroke(bt(VectorBreakout.currentParticles[i].vectorScale));
                    g2.setColor(vc(VectorBreakout.currentParticles[i].particleColor));
                    g2.draw(drawVec(VectorBreakout.currentParticles[i].particleX2, VectorBreakout.currentParticles[i].particleY2));
                }
            }
        }
        
        /* TextElements
         *
         * Note on rendering numbers:
         * 
         * I am aware that I could be using a switch statement for these.
         * However, this did not compile when testing in JDK 1.5, as it would
         * refuse to accept anything other than numbers in a switch statement.
         * Since I am a silly little nerd who wishes to use this on my older
         * systems, I used these goofy if/else statements so I could compile
         * on the older JDKs and play on my older systems...
         */
        for (byte i = 0; i < VectorBreakout.currentTextElements.length; i++) {
            if (VectorBreakout.currentTextElements[i] != null) {
                if (VectorBreakout.currentTextElements[i].active) {
                    resetBeam();
                    moveBeam(VectorBreakout.currentTextElements[i].x, VectorBreakout.currentTextElements[i].y);
                    float oldBeamScaleX = beamScaleX;
                    float oldBeamScaleY = beamScaleY;
                    beamScaleX = beamScaleX * VectorBreakout.currentTextElements[i].textScale;
                    beamScaleY = beamScaleY * VectorBreakout.currentTextElements[i].textScale;
                    g2.setColor(vc(VectorBreakout.currentTextElements[i].textColor));
                    g2.setStroke(bt(VectorBreakout.currentTextElements[i].vectorScale));
                    String stringToSplit = String.valueOf(VectorBreakout.currentTextElements[i].text);
                    String[] stringToRender = stringToSplit.split("");
                    for (short ii = iStart; ii < stringToRender.length; ii++) {
                        if ("1".equals(stringToRender[ii])) {moveBeam(7, 0);g2.draw(drawVec(0, -24));moveBeam(11, 24);}
                        else if ("2".equals(stringToRender[ii])) {moveBeam(0, -24);g2.draw(drawVec(14, 0));g2.draw(drawVec(0, 12));g2.draw(drawVec(-14, 0));g2.draw(drawVec(0, 12));g2.draw(drawVec(14, 0));moveBeam(4, 0);}
                        else if ("3".equals(stringToRender[ii])) {moveBeam(0, -24);g2.draw(drawVec(14, 0));g2.draw(drawVec(0, 12));g2.draw(drawVec(-14, 0));moveBeam(14, 0);g2.draw(drawVec(0, 12));g2.draw(drawVec(-14, 0));moveBeam(18, 0);}
                        else if ("4".equals(stringToRender[ii])) {moveBeam(0, -24);g2.draw(drawVec(0, 12));g2.draw(drawVec(14, 0));moveBeam(0, -12);g2.draw(drawVec(0, 24));moveBeam(4, 0);}
                        else if ("5".equals(stringToRender[ii]) || "S".equals(stringToRender[ii])) {moveBeam(14, -24);g2.draw(drawVec(-14, 0));g2.draw(drawVec(0, 12));g2.draw(drawVec(14, 0));g2.draw(drawVec(0, 12));g2.draw(drawVec(-14, 0));moveBeam(18, 0);}
                        else if ("6".equals(stringToRender[ii])) {moveBeam(0, -24);g2.draw(drawVec(0, 24));g2.draw(drawVec(14, 0));g2.draw(drawVec(0, -12));g2.draw(drawVec(-14, 0));moveBeam(18, 12);}
                        else if ("7".equals(stringToRender[ii])) {moveBeam(0, -24);g2.draw(drawVec(14, 0));g2.draw(drawVec(0, 24));moveBeam(4, 0);}
                        else if ("8".equals(stringToRender[ii])) {g2.draw(drawVec(0, -24));g2.draw(drawVec(14, 0));g2.draw(drawVec(0, 24));g2.draw(drawVec(-14, 0));moveBeam(0, -12);g2.draw(drawVec(14, 0));moveBeam(4, 12);}
                        else if ("9".equals(stringToRender[ii])) {moveBeam(14, 0);g2.draw(drawVec(0, -24));g2.draw(drawVec(-14, 0));g2.draw(drawVec(0, 12));g2.draw(drawVec(14, 0));moveBeam(4, 12);}
                        else if ("0".equals(stringToRender[ii]) || "O".equals(stringToRender[ii])) {g2.draw(drawVec(14, 0));g2.draw(drawVec(0, -24));g2.draw(drawVec(-14, 0));g2.draw(drawVec(0, 24));moveBeam(18, 0);}
                        else if ("\\".equals(stringToRender[ii])) {moveBeam(-(beamX - VectorBreakout.currentTextElements[i].x), 30);} // Newline
                        else if ("A".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -17));
                            g2.draw(drawVec(7, -7));
                            g2.draw(drawVec(7, 7));
                            g2.draw(drawVec(0, 17));
                            moveBeam(-14, -10);
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 10);
                        }
                        else if ("B".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(11, 0));
                            g2.draw(drawVec(3, 3));
                            g2.draw(drawVec(0, 6));
                            g2.draw(drawVec(-3, 3));
                            g2.draw(drawVec(3, 3));
                            g2.draw(drawVec(0, 6));
                            g2.draw(drawVec(-3, 3));
                            g2.draw(drawVec(-11, 0));
                            moveBeam(0, -12);
                            g2.draw(drawVec(11, 0));
                            moveBeam(7, 12);
                        }
                        else if ("C".equals(stringToRender[ii])) {
                            moveBeam(14, -24);
                            g2.draw(drawVec(-14, 0));
                            g2.draw(drawVec(0, 24));
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 0);
                        }
                        else if ("D".equals(stringToRender[ii])) {
                            g2.draw(drawVec(7, 0));
                            g2.draw(drawVec(7, -7));
                            g2.draw(drawVec(0, -10));
                            g2.draw(drawVec(-7, -7));
                            g2.draw(drawVec(-7, 0));
                            g2.draw(drawVec(0, 24));
                            moveBeam(18, 0);
                        }
                        else if ("E".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(14, 0));
                            moveBeam(-14, 12);
                            g2.draw(drawVec(14, 0));
                            moveBeam(-14, 12);
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 0);
                        }
                        else if ("F".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(14, 0));
                            moveBeam(-14, 12);
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 12);
                        }
                        else if ("G".equals(stringToRender[ii])) {
                            moveBeam(14, -17);
                            g2.draw(drawVec(0, -7));
                            g2.draw(drawVec(-14, 0));
                            g2.draw(drawVec(0, 24));
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, -8));
                            g2.draw(drawVec(-8, 0));
                            moveBeam(12, 8);
                        }
                        else if ("H".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            moveBeam(14, 0);
                            g2.draw(drawVec(0, 24));
                            moveBeam(-14, -12);
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 12);
                        }
                        else if ("I".equals(stringToRender[ii])) {
                            g2.draw(drawVec(14, 0));
                            moveBeam(-14, -24);
                            g2.draw(drawVec(14, 0));
                            moveBeam(-7, 0);
                            g2.draw(drawVec(0, 24));
                            moveBeam(11, 0);
                        }
                        else if ("J".equals(stringToRender[ii])) {
                            moveBeam(0, -7);
                            g2.draw(drawVec(7, 7));
                            g2.draw(drawVec(7, 0));
                            g2.draw(drawVec(0, -24));
                            moveBeam(4, 24);
                        }
                        else if ("K".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            moveBeam(14, 0);
                            g2.draw(drawVec(-14, 12));
                            g2.draw(drawVec(14, 12));
                            moveBeam(4, 0);
                        }
                        else if ("L".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(0, 24));
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 0);
                        }
                        else if ("M".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(7, 7));
                            g2.draw(drawVec(7, -7));
                            g2.draw(drawVec(0, 24));
                            moveBeam(4, 0);
                        }
                        else if ("N".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(14, 24));
                            g2.draw(drawVec(0, -24));
                            moveBeam(4, 24);
                        }
                        else if ("P".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, 12));
                            g2.draw(drawVec(-14, 0));
                            moveBeam(18, 12);
                        }
                        else if ("Q".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, 18));
                            g2.draw(drawVec(-6, 6));
                            g2.draw(drawVec(-8, 0));
                            moveBeam(8, -6);
                            g2.draw(drawVec(6, 6));
                            moveBeam(4, 0);
                        }
                        else if ("R".equals(stringToRender[ii])) {
                            g2.draw(drawVec(0, -24));
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, 12));
                            g2.draw(drawVec(-14, 0));
                            moveBeam(2, 0);
                            g2.draw(drawVec(12, 12));
                            moveBeam(4, 0);
                        }
                        else if ("T".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(14, 0));
                            moveBeam(-7, 0);
                            g2.draw(drawVec(0, 24));
                            moveBeam(11, 0);
                        }
                        else if ("U".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(0, 24));
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(0, -24));
                            moveBeam(4, 24);
                        }
                        else if ("V".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(7, 24));
                            g2.draw(drawVec(7, -24));
                            moveBeam(4, 24);
                        }
                        else if ("W".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(0, 24));
                            g2.draw(drawVec(7, -7));
                            g2.draw(drawVec(7, 7));
                            g2.draw(drawVec(0, -24));
                            moveBeam(4, 24);
                        }
                        else if ("X".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(14, 24));
                            moveBeam(-14, 0);
                            g2.draw(drawVec(14, -24));
                            moveBeam(4, 24);
                        }
                        else if ("Y".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(7, 7));
                            g2.draw(drawVec(7, -7));
                            moveBeam(-7, 7);
                            g2.draw(drawVec(0, 17));
                            moveBeam(11, 0);
                        }
                        else if ("Z".equals(stringToRender[ii])) {
                            moveBeam(0, -24);
                            g2.draw(drawVec(14, 0));
                            g2.draw(drawVec(-14, 24));
                            g2.draw(drawVec(14, 0));
                            moveBeam(4, 0);
                        }
                        else if (".".equals(stringToRender[ii])) {
                            moveBeam(7, 0);
                            g2.draw(drawVec(0, -2));
                            moveBeam(11, 2);
                        }
                        else if (",".equals(stringToRender[ii])) {
                            moveBeam(7, 0);
                            g2.draw(drawVec(0, -5));
                            moveBeam(11, 5);
                        }
                        else if (":".equals(stringToRender[ii])) {
                            moveBeam(7, 0);
                            g2.draw(drawVec(0, -2));
                            moveBeam(0, -10);
                            g2.draw(drawVec(0, -2));
                            moveBeam(11, 14);
                        }
                        else if (";".equals(stringToRender[ii])) {
                            moveBeam(7, 0);
                            g2.draw(drawVec(0, -5));
                            moveBeam(0, -7);
                            g2.draw(drawVec(0, -2));
                            moveBeam(11, 14);
                        }
                        else if ("!".equals(stringToRender[ii])) {
                            moveBeam(7, 0);
                            g2.draw(drawVec(0, -2));
                            moveBeam(0, -10);
                            g2.draw(drawVec(0, -12));
                            moveBeam(11, 24);
                        }
                        else { // Blank space
                            moveBeam(18, 0);
                        }
                    }
                    beamScaleX = oldBeamScaleX;
                    beamScaleY = oldBeamScaleY;
                }
            }
        }
        
        // Paused
        g2.setStroke(bt(3));
        if (VectorBreakout.isPaused) {
            g2.setColor(Color.white);
            resetBeam();
            moveBeam(110, 260);
            // P
            g2.draw(drawVec(0, -110));g2.draw(drawVec(80, 0));g2.draw(drawVec(0, 50));g2.draw(drawVec(-80, 0));
            moveBeam(100, 60);
            // A
            g2.draw(drawVec(0, -70));g2.draw(drawVec(40, -40));g2.draw(drawVec(40, 40));g2.draw(drawVec(0, 70));g2.draw(drawVec(0, -40));g2.draw(drawVec(-80, 0));
            moveBeam(100, 40);
            // U
            moveBeam(0, -110);g2.draw(drawVec(0, 110));g2.draw(drawVec(80, 0));g2.draw(drawVec(0, -110));
            moveBeam(20, 110);
            // S
            g2.draw(drawVec(80, 0));g2.draw(drawVec(0, -55));g2.draw(drawVec(-80, 0));g2.draw(drawVec(0, -55));g2.draw(drawVec(80, 0));
            moveBeam(20, 110);
            // E
            moveBeam(80, 0);g2.draw(drawVec(-80, 0));g2.draw(drawVec(0, -55));g2.draw(drawVec(80, 0));moveBeam(-80, 0);g2.draw(drawVec(0, -55));g2.draw(drawVec(80, 0));
            moveBeam(20, 110);
            // D
            g2.draw(drawVec(40, 0));g2.draw(drawVec(40, -40));g2.draw(drawVec(0, -30));g2.draw(drawVec(-40, -40));g2.draw(drawVec(-40, 0));g2.draw(drawVec(0, 110));
            moveBeam(100, 110);
        }
        
        VectorBreakout.actualFrames++; // Count up actual frames rendered
        
        // Debug Menu
        if (VectorBreakout.debugMenuEnabled) {
            g2.setColor(Color.white);
            g2.drawString("Quajra Debug", 1, 12);
            g2.drawString("frame: " + VectorBreakout.frameCounter, 1, 24);
            g2.drawString("beamScaleX: " + beamScaleX, 1, 36);
            g2.drawString("beamScaleY: " + beamScaleY, 1, 48);
            g2.drawString("isPaused: " + VectorBreakout.isPaused, 1, 60);
            g2.drawString("targetFrameRate: " + VectorBreakout.targetFrameRate, 150, 24);
            g2.drawString("actualFrameRate: " + VectorBreakout.actualFrameRate, 150, 36);
            g2.drawString("actualFrames: " + VectorBreakout.actualFrames, 150, 48);
            g2.drawString("chancey: " + VectorBreakout.chancey, 150, 60);
            g2.drawString("ballsOnScreen: " + VectorBreakout.ballsOnScreen, 300, 24);
        }
        
    }
    
    public Line2D.Float drawVec(float x, float y) {
        float oldBeamX = beamX;
        float oldBeamY = beamY;
        beamX += (x * beamScaleX);
        beamY += (y * beamScaleY);
        return new Line2D.Float(oldBeamX, oldBeamY, beamX, beamY);
    }
    
    public Color vc(int col) {
        switch ((byte) col) {
            case 0: return Color.red;
            case 1: return Color.getHSBColor(25, 258, 77);
            case 2: return Color.cyan;
            case 3: return Color.green;
            case 4: return Color.magenta;
            case 5: return Color.yellow;
            case 15: return Color.white;
            case 16: return Color.getHSBColor(313, 34, 100); // pink test
            default: return Color.white;
        }
    }
    
    public BasicStroke bt(float size) {
        return new BasicStroke(size * beamThicknessScale);
    }
    
    public void moveBeam(float x, float y) {
        beamX += (x * beamScaleX);
        beamY += (y * beamScaleY);
    }
    
    public void resetBeam() {
        beamX = 0;
        beamY = 40 * beamScaleY;
    }
    
    public GameDisplay() {
    }
    
}
